﻿using UnityEngine;
using System.Collections;

public enum MoveDirection {
	HORIZONTAL,
	VERTICAL,
	NONE
}

public struct MoveData {
	public MoveDirection direction;
	public int distance;

	public MoveData(MoveDirection direction, int distance) {
		this.direction = direction;
		this.distance = distance;
	}
}

public class Character : MonoBehaviour {
	public float moveSpeed;	
	public StageTile currentTile;
	public StageTile goalTile;
	public StageHandler stageHandler;
	public bool activeControl;
	public Camera childCamera;
	public Character partner;
	public GameObject blindFog;
	public bool follow;
	public bool main;
	public bool dead;
	public GameObject activeMark;
	public GameObject potionMark;
	public GameObject senseMark;
	public int detectedNum;

	private tk2dSprite sprite;
	private tk2dSpriteAnimator anim;
	private MoveDirection direction;

	public const float MOVE_TOLERANCE = 0.0001f;
	public const float PARTNER_FOLLOW_DISTANCE = 0.8f;
	public const float PARTNER_STOP_DISTANCE = 0.2f;
	public const float ZPOS_FRONT = -1.0f;
	public const float ZPOS_BACK = -0.5f;

	void Awake() {
		this.direction = MoveDirection.NONE;
		this.sprite = this.GetComponent<tk2dSprite>();
		this.anim = this.GetComponent<tk2dSpriteAnimator>();

		if(this.main) { 
			this.childCamera = this.GetComponentInChildren<Camera>();
		}
	}

	// Use this for initialization
	void Start () {
		MoveData moveData = CheckDirection();
		Move(moveData);
	}

	// Update is called once per frame
	void Update () {
		if(this.dead) {
			return;
		}

		if(this.activeControl) {
			if(this.goalTile && !this.goalTile.locked) {
				Vector3 charPos = this.transform.position;
				Vector3 goalPos = this.goalTile.transform.position;

				if(Vector2.Distance((Vector2)charPos, (Vector2)goalPos) < MOVE_TOLERANCE) {
					this.transform.position = new Vector3(goalPos.x, goalPos.y, charPos.z);
					this.currentTile = this.goalTile;
				}
				else {
					this.transform.position = Vector3.MoveTowards(charPos, new Vector3(goalPos.x, goalPos.y, charPos.z), Time.deltaTime * this.moveSpeed);
					return;
				}
			}

			MoveData moveData = CheckDirection();


			if(moveData.direction == MoveDirection.HORIZONTAL) {
				if(moveData.distance > 0) {
					this.sprite.FlipX = true;
				}
				else {
					this.sprite.FlipX = false;
				}

				PlayAnim("WalkSide");
			}
			else if(moveData.direction == MoveDirection.VERTICAL) {
				if(moveData.distance > 0) {
					PlayAnim("WalkDown");
				}
				else {
					PlayAnim("WalkUp");
				}
			}
			else {
				PlayAnim("Idle");
			}

			Move(moveData);
		}
		else {
			if(follow) {
				if(Vector2.Distance((Vector2)this.transform.position, (Vector2)this.partner.transform.position) > PARTNER_STOP_DISTANCE) {
					this.currentTile = this.partner.currentTile;
					Vector3 partnerPos = this.partner.transform.position;
					this.transform.position = Vector3.MoveTowards(this.transform.position, new Vector3(partnerPos.x, partnerPos.y, this.transform.position.z), Time.deltaTime * this.moveSpeed);
				
					if(this.partner.direction == MoveDirection.HORIZONTAL) {
						if(this.transform.position.x < this.partner.transform.position.x) {
							this.sprite.FlipX = true;
						}
						else {
							this.sprite.FlipX = false;
						}
						
						PlayAnim("WalkSide");
					}
					else if(this.partner.direction == MoveDirection.VERTICAL) {
						if(this.transform.position.y > this.partner.transform.position.y) {
							PlayAnim("WalkDown");
						}
						else {
							PlayAnim("WalkUp");
						}
					}
					else {
						PlayAnim("Idle");
					}
				}
				else {
					this.follow = false;

					if(!this.anim.IsPlaying("Idle")) {
						this.anim.Play("Idle");
					}
				}
			}
			else {
				if(Vector2.Distance((Vector2)this.transform.position, (Vector2)this.partner.transform.position) > PARTNER_FOLLOW_DISTANCE) {
					this.follow = true;
				}
			}
		}
	}

	public void AddDetect() {
		if(this.main) {
			return;
		}

		this.detectedNum++;
		this.senseMark.SetActive(true);
		StageHandler.PlayClip((AudioClip)Resources.Load("Audio/spider sense"));
	}

	public void SubDetect() {
		if(this.main) {
			return;
		}

		this.detectedNum--;
		this.detectedNum = this.detectedNum <= 0 ? 0 : this.detectedNum;

		if(this.detectedNum == 0) {
			this.senseMark.SetActive(false);
		}
	}

	private MoveData CheckDirection() {
		float horz = Input.GetAxis("Horizontal");
		float vert = Input.GetAxis("Vertical");
		int distance = 0;
		MoveDirection direction = MoveDirection.NONE;

		if(horz == 0 && vert == 0) {
			return new MoveData(direction, distance);
		}

		if(Mathf.Abs(horz) > Mathf.Abs(vert)) {
			distance = horz < 0 ? -1 : 1;
			direction = MoveDirection.HORIZONTAL;
		}
		else {
			distance = vert < 0 ? 1 : -1;
			direction = MoveDirection.VERTICAL;
		}

		return new MoveData(direction, distance);
	}

	private void Move(MoveData moveData) {
		this.direction = moveData.direction;

		if(this.direction == MoveDirection.HORIZONTAL) {
			this.goalTile = this.stageHandler.GetTile(this.currentTile.row, this.currentTile.col + moveData.distance);
		}
		else {
			this.goalTile = this.stageHandler.GetTile(this.currentTile.row + moveData.distance, this.currentTile.col);
		}
	}

	public void SetViewActive(bool active) {
		this.activeControl = active;
		this.activeMark.SetActive(active);

		if(active) {
			this.transform.position = new Vector3(this.transform.position.x, this.transform.position.y, ZPOS_FRONT);

			if(this.main) {
				this.blindFog.SetActive(!active);
				this.childCamera.cullingMask = (1 << LayerMask.NameToLayer("See")) | (1 << LayerMask.NameToLayer("Default"));
			}
		}
		else {
			this.transform.position = new Vector3(this.transform.position.x, this.transform.position.y, ZPOS_BACK);

			if(this.main) {
				this.blindFog.SetActive(!active);
				this.childCamera.cullingMask = (1 << LayerMask.NameToLayer("Blind")) | (1 << LayerMask.NameToLayer("Default"));
			}
		}
	}

	public void PlayAnim(string clipName) {
		if(!this.anim.IsPlaying(clipName)) {
			this.anim.Play(clipName);
		}
	}

	public void CarryPotion() {
		this.potionMark.SetActive(true);
	}
}
